/*
  ==============================================================================

   This file is part of the JUCE library - "Jules' Utility Class Extensions"
   Copyright 2004-7 by Raw Material Software ltd.

  ------------------------------------------------------------------------------

   JUCE can be redistributed and/or modified under the terms of the
   GNU General Public License, as published by the Free Software Foundation;
   either version 2 of the License, or (at your option) any later version.

   JUCE is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with JUCE; if not, visit www.gnu.org/licenses or write to the
   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
   Boston, MA 02111-1307 USA

  ------------------------------------------------------------------------------

   If you'd like to release a closed-source product which uses JUCE, commercial
   licenses are also available: visit www.rawmaterialsoftware.com/juce for
   more information.

  ==============================================================================
*/

#include <extras/the jucer/src/jucer_Headers.h>
#include "jucer_FilePropertyComponent.h"


//==============================================================================
FilePropertyComponent::FilePropertyComponent (const String& name,
                                              const bool isDirectory,
                                              const bool allowEditingOfFilename,
                                              const String& fileBrowserWildcard)
    : PropertyComponent (name)
{
    addAndMakeVisible (filenameComp = new FilenameComponent (name,
                                                             File::nonexistent,
                                                             allowEditingOfFilename,
                                                             isDirectory,
                                                             false,
                                                             fileBrowserWildcard,
                                                             String::empty,
                                                             String::empty));

    filenameComp->addListener (this);
}

FilePropertyComponent::~FilePropertyComponent()
{
    deleteAllChildren();
}

void FilePropertyComponent::refresh()
{
    filenameComp->setCurrentFile (getFile(), false);
}

void FilePropertyComponent::filenameComponentChanged (FilenameComponent*)
{
    if (getFile() != filenameComp->getCurrentFile())
        setFile (filenameComp->getCurrentFile());
}
