//
// logger_component.h
//
// Author:  <>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef NTK_GUI_logger_component_H
# define NTK_GUI_logger_component_H

# include <ntk/core.h>
# include "per_thread_logger.h"

class LoggerComponent : public TextEditor, public ThreadLogger, public AsyncUpdater
{
  public:
    LoggerComponent(const String& initial_text)
  : TextEditor()
    {
      setMultiLine (true);
      setReturnKeyStartsNewLine (false);
      setReadOnly (true);
      setScrollbarsShown (true);
      setCaretVisible (true);
      setPopupMenuEnabled (true);
      setColour(backgroundColourId, Colours::lightgrey);
      logMessage(initial_text);
    }
    
  juce_UseDebuggingNewOperator

  protected:
    virtual void logMessage(const String& message)
    {
      const ScopedLock lock(m_critical_section);
      m_current_message += message + JUCE_T("\n");
      triggerAsyncUpdate();
    }
    
  protected:
    virtual void handleAsyncUpdate()
    {
      const ScopedLock lock(m_critical_section);
      insertTextAtCursor(m_current_message);
      m_current_message = String::empty;
    }
    
  private:
    String m_current_message;
    CriticalSection m_critical_section;
};

#endif // ndef NTK_GUI_logger_component_H
