/*
** hypothesis_testing.h
** Login : <nicolas.burrus@ensta.fr>
** Started on  Thu Jul 27 15:02:40 2006 Nicolas Burrus
** $Id$
** 
** Copyright (C) 2006, 2007 Nicolas Burrus
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef   	NTK_STATS_HYPOTHESIS_TESTING_H_
# define   	NTK_STATS_HYPOTHESIS_TESTING_H_

# include <ntk/core.h>
# include <vcl_cmath.h>
# include <vcl_vector.h>
# include <vcl_map.h>

namespace ntk
{

  /**
   * Algorithms and examples come from "Non-parametric Statistics", Sigel,
   * McGraw-Hill Series in Psychology, 1956
   */

  vcl_vector< vcl_vector< vcl_vector<double> > > generate_mann_whitney_values();
  
  template <class T>
  double compute_mann_whitney_log_pfa(const vcl_vector<T>& d1, 
                                      const vcl_vector<T>& d2,
                                      double n1 = -1, double n2 = -1);
 
  /**
   * Probability that a samples of n element get a deviation dev by chance 
   * while underlying distribution moments are gdev and gmu4.
   */
  double deviation_log_pfa(double dev, double n, double gdev, double gmu4);

  template <class cdf_function>
  double cramer_von_mises_nw2(const vcl_map<double,double>& distrib, cdf_function f);

  template <class cdf_function>
  double kolmogorov_sqrt_n_Dn(const vcl_map<double,double>& distrib, cdf_function f);
  
} // end of ntk

# include "hypothesis_testing.hxx"

#endif 	    /* !NTK_STATS_HYPOTHESIS_TESTING_H_ */
