//
// hypothesis_testing.hxx
// 
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

# include <vcl_algorithm.h>

namespace ntk
{

  template <class cdf_function>
  double cramer_von_mises_nw2(const vcl_map<double,double>& distrib, cdf_function f)
  {
    typedef vcl_map<double,double> distrib_type;
    double n = 0;
    foreach_const_it(it, distrib, distrib_type) n += it->second;
    
    double v = 0;
    double i = 1;
    foreach_const_it(it, distrib, distrib_type)
    {
      double cdf_it = f(it->first);
      for (int j = 0; j < it->second; ++j)
      {
        double t = ((2.0*i-1)/(2.0*n)) - cdf_it;
        v += (t*t);
        i++;
      }
    }
    v += (1.0/(12*n));
    return v;
  }
  
  template <class cdf_function>
  double kolmogorov_sqrt_n_Dn(const vcl_map<double,double>& distrib, cdf_function f)
  {
    typedef vcl_map<double,double> distrib_type;
    
    double n = 0;
    foreach_const_it(it, distrib, distrib_type)
      n += it->second;
    
    double sup = 0;
    double cur_cdf = 0;
    foreach_const_it(it, distrib, distrib_type)
    {
      double t1 = vcl_abs(cur_cdf - f(it->first));
      cur_cdf += (it->second/n);
      double t2 = vcl_abs(cur_cdf - f(it->first));
      sup = vcl_max(sup, t1);
      sup = vcl_max(sup, t2);
    }
    return sqrt(n)*sup;
  }

} // end of ntk

