//
// process.cc
//
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#include "process.h"
#include "debug.h"

#ifndef JUCE_WIN32

extern char** environ;

namespace ntk
{

  int shell_command (const String& pathAndArguments, bool wait_for_terminaison)
  {
    char sh_command[] = "/bin/sh";
    char sh_option[] = "-c";
    char* const argv[4] = { sh_command, sh_option, (char*) (const char*) pathAndArguments, 0 };

    const pid_t cpid = fork();

    if (cpid == 0)
    {
        // Child process
      if (execve (argv[0], argv, environ) < 0)
        exit (0);
    }
    else
    {
      if (cpid < 0)
        return -1;
    }

    if (wait_for_terminaison)
    {
      int status;
      while (wait(&status) != cpid)
        ;
      if (WIFEXITED(status)) 
      {
        ntk_dbg(0) << "Program exited normally";
        return WEXITSTATUS(status);
      }
      return -1;
    }
    return 0;
  }

} // end of ntk

#else

namespace ntk
{
	
  int shell_command (const String& pathAndArguments, bool wait_for_terminaison)
  {
	ntk_assert(0, "not implemented on Win32");
	return 0;
  }

}

#endif
