/*
** test-estimation.cc
** Login : <nicolas.burrus@ensta.fr>
** Started on  Wed Oct 18 12:51:34 2006 Nicolas Burrus
** $Id$
** 
** Copyright (C) 2006 Nicolas Burrus
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <ntk/utils/debug.h>
#include <ntk/utils/taskpool.h>
#include <ntk/vxl/all.h>
#include <ntk/stats/estimation.h>
#include <ntk/stats/distributions.h>

#include <vnl/vnl_random.h>
#include <QFile>
#include <QTextStream>

using namespace ntk;

int main(int argc, char** argv)
{
  if (argc != 2)
  {
    std::cerr << "Usage: " << argv[0] << " plotfile" << std::endl;
    exit(1);
  }

  QFile plot_file (argv[1]);
  if (!plot_file.open(QIODevice::ReadOnly))
  {
    std::cerr << "Could not open " << argv[1] << std::endl;
    exit(2);
  }
  
  QMap<double,double> distrib;
  QTextStream stream (&plot_file);
  while (!stream.atEnd())
  {
    double x, y;
    stream >> x >> y;
    if (x > 0 && y > 0)
      distrib[x] += y;
  }

  WeibullEstimates estimates; 
  estimates.beta = 20; estimates.etha = 2; estimates.gamma = 0;
  estimate_weibull(distrib, estimates);
  ntk_dbg_print(estimates.beta, 0);
  ntk_dbg_print(estimates.etha, 0);
  ntk_dbg_print(wblinv(0.99995, estimates.beta, estimates.etha), 0);
  WeibullEstimates bounds = estimate_weibull_upper_bounds(distrib, estimates, 
                                                          0.99995, 0.99995);
  ntk_dbg_print(bounds.beta, 0);
  ntk_dbg_print(bounds.etha, 0);
  ntk_dbg_print(wblinv(0.99995, bounds.beta, bounds.etha), 0);
}
