//
// ac_explorer.h
//
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef ACSEGMENTOR_ac_explorer_H
# define ACSEGMENTOR_ac_explorer_H

# include <ntk/core.h>

# include "ac_selector.h"

# include <boost/multi_index_container.hpp>
# include <boost/multi_index/ordered_index.hpp>
# include <boost/multi_index/hashed_index.hpp>
# include <boost/multi_index/identity.hpp>
# include <boost/multi_index/mem_fun.hpp>
# include <boost/multi_index/member.hpp>
# include <boost/multi_index/composite_key.hpp>

using namespace boost;
using namespace boost::multi_index;

class Segmentor; ntk_ptr_typedefs(Segmentor);
class AcSegmentor; ntk_ptr_typedefs(AcSegmentor);
class Region;

class AcExplorer
{
  public:
    AcExplorer() {}
    virtual ~AcExplorer () {}

    virtual String name() const = 0;

  public:
    virtual void initialize(AcSelector& selector) { m_selector = &selector; }
    virtual void run() = 0;

  protected:
    AcSegmentor& segmentor() { return m_selector->segmentor(); }
    const AcSegmentor& segmentor() const { return m_selector->segmentor(); }
    AcSelector& selector() { return *m_selector; }
    const AcSelector& selector() const { return *m_selector; }

  private:
    AcSelector* m_selector;
};

class MergeBestExplorer : public AcExplorer
{
  private:
    typedef multi_index_container<
        RegionEdgeWithScore,
        indexed_by<
            hashed_unique<RegionEdgeLabelKey::type>,
            ordered_non_unique<RegionEdgeMeaningfulThenScoreKey::type>
        >
    > edge_set_type;

    typedef edge_set_type::nth_index<0>::type edge_set_by_label_type;
    edge_set_by_label_type& getByLabel(edge_set_type& s) { return s.get<0>(); }

    typedef edge_set_type::nth_index<1>::type edge_set_by_score_type;
    edge_set_by_score_type& getByScore(edge_set_type& s) { return s.get<1>(); }

  public:
    MergeBestExplorer(const SegmentorPtr& initializer)
      : m_initializer(initializer),
        m_keep_all_meaningful_links(true),
        m_max_regions(-1)
    {}

    virtual String name() const;

  public:
    void setMaxNumOfRegions(int n) { m_max_regions = n; }
    void setKeepAllMeaningfulLinks(bool b) { m_keep_all_meaningful_links = b; }

  public:
    virtual void initialize(AcSelector& selector);
    virtual void run();

  private:
    void mergeRegions(const RegionEdgeWithScore& edge);

  private:
    void addEdge(uint32 r1, uint32 r2);
    void removeEdgesTo(const Region& r);
    void buildEdges();
    uint32 getRealRegionId(uint32 r);
    
  private:
    SegmentorPtr m_initializer;
    edge_set_type m_edges;
    vcl_list<RegionEdge> m_regions_to_merge;
    vcl_vector<uint32> m_regions_correspondances;
    bool m_keep_all_meaningful_links;
    int m_max_regions;
};

#endif // ndef ACSEGMENTOR_ac_explorer_H
