/*
  ==============================================================================

  This is an automatically generated file created by the Jucer!

  Creation date:  24 Apr 2008 3:25:49 pm

  Be careful when adding custom code to these files, as only the code within
  the "//[xyz]" and "//[/xyz]" sections will be retained when the file is loaded
  and re-saved.

  Jucer version: 1.11

  ------------------------------------------------------------------------------

  The Jucer is part of the JUCE library - "Jules' Utility Class Extensions"
  Copyright 2004-6 by Raw Material Software ltd.

  ==============================================================================
*/

//[Headers] You can add your own extra header files here...
#include "../segmentor_factory.h"
#include "../ac_segmentor.h"
#include <ntk/ntk_with_macros.h>
//[/Headers]

#include <ntk/ntk_with_macros.h>
#include "segmentation_main_component.h"


//[MiscUserDefs] You can add your own user definitions and misc code here...
using namespace ntk;

class SegmentationJob : public Thread, public AsyncUpdater, public ChangeListener
{
  public:
    SegmentationJob(const PropertySetSubject& properties,
                    SegmentationJobListener* listener)
      : Thread("Segmentation"), m_properties(properties), m_listener(listener)
    {
      SegmentorFactory factory;
      try {
        m_segmentor = factory.createSegmentor(m_properties);
      }
      catch (const vcl_exception& e)
      {
        m_error_message = String(T("Could not create segmentor `"))
                          << e.what() << "'";
        return;
      }
      if (!m_segmentor)
      {
        m_error_message = T("Could not load create segmentor.");
        return;
      }

      String image = m_properties.getValue("image_to_segment");
      m_source_image = vil_load(image.toUTF8());
      if (!m_source_image)
      {
        m_error_message = String("Could not load image: `") << image << "'";
        return;
      }
    }

  public:
    const String& getErrorMessage() const { return m_error_message; }
    const vil_byte_image& getSourceImage() const { return m_source_image; }
    const vil_byte_image& getSegmentedImage(bool with_edges) const
    {
      if (with_edges) return m_edges_image;
      return m_mean_image;
    }
    SegmentorPtr getSegmentor() const { return m_segmentor; }

  public:
    bool isValid() const { return m_source_image && m_segmentor; }

    virtual void run()
    {
      if (!isValid())
        return stopWithError("No segmentor or no source image");

      m_finished = false;
      m_segmentor->setRefreshRate(-1);
      m_segmentor->addChangeListener(this);
      try
      {
        m_segmentor->segment(m_source_image);
        m_segmentor->getMeanImage(m_mean_image);
        ntk_dbg_print(m_segmentor->nbRegions(), 1);

        if (m_properties.getValue(T("output")).isNotEmpty())
          vil_save(m_mean_image, m_properties.getValue("output"));

        m_segmentor->getMeanImage(m_edges_image, true); // with borders.
        if (m_properties.getValue(T("output-edges")).isNotEmpty())
          vil_save(m_edges_image, m_properties.getValue("output-edges"));
      }
      catch (const vcl_exception& e)
      {
        return stopWithError(String("Error !") << e.what());
      }
      m_finished = true;
      triggerAsyncUpdate();
    }

  protected:
    void stopWithError(const String& message)
    {
      m_finished = true;
      m_error_message = message;
      triggerAsyncUpdate();
    }

    void handleAsyncUpdate()
    {
      if (m_finished)
        m_listener->handleSegmentationTerminated(this);
      else
        m_listener->handleSegmentationUpdated(this);
    }

    virtual void changeListenerCallback (void *objectThatHasChanged)
    {
      triggerAsyncUpdate();
    }

  private:
    SegmentorPtr m_segmentor;
    const PropertySetSubject& m_properties;
    SegmentationJobListener* m_listener;
    vil_byte_image m_mean_image;
    vil_byte_image m_edges_image;
    vil_byte_image m_source_image;
    String m_error_message;
    bool m_finished;
};

//[/MiscUserDefs]

//==============================================================================
SegmentationMainComponent::SegmentationMainComponent (const PropertySetSubject& properties)
    : m_properties(properties),
      m_source_image (0),
      m_result_image (0),
      m_logger (0),
      m_initial_segmentation (0)
{
  //[Constructor_pre] You can add your own custom stuff here..
  //[/Constructor_pre]

  addAndMakeVisible (m_source_image = new ImageComponent());
  m_source_image->setName (T("source component"));

  addAndMakeVisible (m_result_image = new ImageComponent());
  m_result_image->setName (T("result component"));

  addAndMakeVisible (m_logger = new LoggerComponent ("[Launching segmentation]"));
  m_logger->setName (T("logger"));

  addAndMakeVisible (m_initial_segmentation = new ImageComponent());
  m_initial_segmentation->setName (T("initial segmentation image"));


  //[UserPreSize]
  //[/UserPreSize]

  setSize (600, 400);

  //[Constructor] You can add your own custom stuff here..
  //[/Constructor]
}

SegmentationMainComponent::~SegmentationMainComponent()
{
  //[Destructor_pre]. You can add your own custom destruction code here..
  //[/Destructor_pre]

  deleteAndZero (m_source_image);
  deleteAndZero (m_result_image);
  deleteAndZero (m_logger);
  deleteAndZero (m_initial_segmentation);

  //[Destructor]. You can add your own custom destruction code here..
  //[/Destructor]
}

//==============================================================================
void SegmentationMainComponent::paint (Graphics& g)
{
    //[UserPrePaint] Add your own custom painting code here..
    //[/UserPrePaint]

    g.fillAll (Colours::white);

    //[UserPaint] Add your own custom painting code here..
    //[/UserPaint]
}

void SegmentationMainComponent::resized()
{
    m_source_image->setBounds (0, 0, proportionOfWidth (0.3330f), getHeight() - 100);
    m_result_image->setBounds (((0) + (proportionOfWidth (0.3330f))) + (proportionOfWidth (0.3333f)), 0, proportionOfWidth (0.3333f), getHeight() - 100);
    m_logger->setBounds (0, getHeight() - 100, proportionOfWidth (1.0000f), 100);
    m_initial_segmentation->setBounds ((0) + (proportionOfWidth (0.3330f)), 0, proportionOfWidth (0.3333f), getHeight() - 100);
    //[UserResized] Add your own custom resize handling here..
    // FIXME: automatically handle border.
#if 0
    const int padding = 2;
    const int half_padding = 1;
    m_source_image->setBounds (padding, padding,
                               proportionOfWidth (0.5000f)-half_padding-padding,
                               getHeight() - 100 - padding-half_padding);
    m_result_image->setBounds ((getWidth() / 2) + half_padding, padding,
                               proportionOfWidth (0.5000f)-half_padding-padding,
                               getHeight() - 100 - padding-half_padding);
    m_logger->setBounds (padding, getHeight() - 100 + half_padding,
                         proportionOfWidth (1.0000f) - 2*padding, 100-padding-half_padding);
#endif
    //[/UserResized]
}



//[MiscUserCode] You can add your own definitions of your custom methods or any other code here...
bool SegmentationMainComponent :: startSegmentation()
{  
  m_first_update = true;
  SegmentationJob* job = new SegmentationJob(m_properties, this);
  if (job->isValid())
  {
    m_source_image->setImage(vil_to_juce(job->getSourceImage()));
    PerThreadLogger::getInstance()->registerLogger(job, m_logger);
    job->startThread();
    return true;
  }
  else
  {
    AlertWindow::showMessageBox(AlertWindow::WarningIcon,
                                "Error !",
                                job->getErrorMessage(),
                                "Ok");
    delete job;
  }
  return false;
}

void SegmentationMainComponent :: handleSegmentationTerminated(SegmentationJob* job)
{
  if (job->getErrorMessage().isNotEmpty())
  {
    AlertWindow::showMessageBox(AlertWindow::WarningIcon,
                                "Error !",
                                job->getErrorMessage(),
                                "Ok");
  }
  else
  {
    m_result_image->setImage(vil_to_juce(job->getSegmentedImage(true)));
  }
  PerThreadLogger::getInstance()->removeLogger(job);
  delete job;
}

void SegmentationMainComponent :: handleSegmentationUpdated(SegmentationJob* job)
{
  vil_byte_image im = job->getSegmentor()->getCurrentSegmentation();
  if (!im) return;
  m_result_image->setImage(vil_to_juce(im));
  if (m_first_update)
  {
    m_initial_segmentation->setImage(vil_to_juce(im));
    m_first_update = false;
  }
}
//[/MiscUserCode]


//==============================================================================
#if 0
/*  -- Jucer information section --

  This is where the Jucer puts all of its metadata, so don't change anything in here!

BEGIN_JUCER_METADATA

<JUCER_COMPONENT documentType="Component" className="SegmentationMainComponent"
                 componentName="" parentClasses="public Component, public SegmentationJobListener"
                 constructorParams="const PropertySetSubject&amp; properties" variableInitialisers="m_properties(properties)"
                 snapPixels="8" snapActive="1" snapShown="1" overlayOpacity="0.330000013"
                 fixedSize="0" initialWidth="600" initialHeight="400">
  <BACKGROUND backgroundColour="ffffffff"/>
  <GENERICCOMPONENT name="source component" id="aadf8a554cdc5c67" memberName="m_source_image"
                    virtualName="" explicitFocusOrder="0" pos="0 0 33.333% 100M"
                    class="ImageComponent" params=""/>
  <GENERICCOMPONENT name="result component" id="96a53339ed66ee17" memberName="m_result_image"
                    virtualName="" explicitFocusOrder="0" pos="0R 0 33.333% 100M"
                    posRelativeX="e7458b7b1e5a47fb" class="ImageComponent" params=""/>
  <GENERICCOMPONENT name="logger" id="33000e36bea3fd06" memberName="m_logger" virtualName=""
                    explicitFocusOrder="0" pos="0 0Rr 100% 100" class="LoggerComponent"
                    params="&quot;[Launching segmentation]&quot;"/>
  <GENERICCOMPONENT name="initial segmentation image" id="e7458b7b1e5a47fb" memberName="m_initial_segmentation"
                    virtualName="" explicitFocusOrder="0" pos="0R 0 33.333% 100M"
                    posRelativeX="aadf8a554cdc5c67" class="ImageComponent" params=""/>
</JUCER_COMPONENT>

END_JUCER_METADATA
*/
#endif
