//
// segmentor_factory.h
//
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef ACSEGMENTOR_segmentor_factory_H
# define ACSEGMENTOR_segmentor_factory_H

# include <ntk/core.h>

class Segmentor; ntk_ptr_typedefs(Segmentor);
class ExternalSegmentor; ntk_ptr_typedefs(ExternalSegmentor);
class AcSegmentor; ntk_ptr_typedefs(AcSegmentor);
class AcSelector; ntk_ptr_typedefs(AcSelector);
class AcExplorer; ntk_ptr_typedefs(AcExplorer);
class AcLearnProcess; ntk_ptr_typedefs(AcLearnProcess);

class SegmentorFactory
{
  public:
    SegmentorPtr createSegmentor(const PropertySet& properties);
    AcLearnProcessPtr createLearnProcess(const PropertySet& properties);
    static void getDefaultProperties(PropertySet& properties);

  public:
    const AcSelectorPtr& acSelector() const { return m_ac_selector; }
    const AcSegmentorPtr& acSegmentor() const { return m_ac_segmentor; }

  private:
    SegmentorPtr createEgbisSegmentor(const PropertySet& properties);
    SegmentorPtr createWatershedSegmentor(const PropertySet& properties);
    SegmentorPtr createSrmSegmentor(const PropertySet& properties);
    SegmentorPtr createEdisonSegmentor(const PropertySet& properties);
    AcSegmentorPtr createAcSegmentor(const PropertySet& properties);
    AcSelectorPtr createAcSelector(const PropertySet& properties);
    AcExplorerPtr createAcExplorer(const PropertySet& properties);

    void clearAll()
    { m_ac_selector.reset(); }

  private:
    AcSegmentorPtr m_ac_segmentor;
    AcSelectorPtr m_ac_selector;
};

#endif // ndef ACSEGMENTOR_segmentor_factory_H
